<?php

/**
 * @file
 * Contains \Drupal\smallads\Entity\SmallAd.
 *
 */

namespace Drupal\smallads\Entity;

use Drupal\user\EntityOwnerInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;
use Drupal\Core\Entity\EntityStorageInterface;
//@todo is this 'use' needed
use Drupal\Core\Datetime\DrupalDateTime;

/**
 * Defines the SmallAd entity.
 *
 * @ContentEntityType(
 *   id = "smallad",
 *   label = @Translation("Small Ad"),
 *   handlers = {
 *     "storage" = "Drupal\smallads\SmallAdStorage",
 *     "view_builder" = "Drupal\smallads\SmallAdViewBuilder",
 *     "list_builder" = "Drupal\smallads\SmallAdListBuilder",
 *     "access" = "Drupal\smallads\SmallAdAccessControlHandler",
 *     "form" = {
 *       "edit" = "Drupal\smallads\Form\SmallAdEdit",
 *       "delete" = "Drupal\smallads\Form\SmallAdDeleteConfirm",
 *     },
 *     "views_data" = "Drupal\smallads\SmallAdViewsData",
 *     "route_provider" = {
 *       "html" = "Drupal\smallads\Entity\SmallAdRouteProvider",
 *     },
 *   },
 *   admin_permission = "configure smallads",
 *   base_table = "smallad",
 *   data_table = "smallad_field_data",
 *   translatable = TRUE,
 *   entity_keys = {
 *     "id" = "smid",
 *     "uuid" = "uuid",
 *     "label" = "strapline",
 *     "langcode" = "langcode",
 *   },
 *   field_ui_base_route = "smallads.admin",
 *   translatable = TRUE,
 *   links = {
 *     "canonical" = "/ad/{smallad}",
 *     "edit-form" = "/ad/{smallad}/edit",
 *     "delete-form" = "/ad/{smallad}/delete",
 *     "collection" = "/admin/structure/smallads/list",
 *   }
 * )
 */
class SmallAd extends ContentEntityBase implements EntityOwnerInterface {

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    //note that the worth field is field API because we can't define multiple cardinality fields in this entity.
    $fields['smid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Ad ID'))
      ->setDescription(t('Locally unique smallAd database key.'))
      ->setReadOnly(TRUE)
      ->setSetting('unsigned', TRUE);

    //seems to be good practice and cheap to have this
    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('Probably globally unique UUID.'))
      ->setReadOnly(TRUE);

    $fields['langcode'] = BaseFieldDefinition::create('language')
      ->setLabel(t('Language'))
      ->setDescription(t('The node language code.'))
      ->setTranslatable(TRUE)
      ->setRevisionable(TRUE)
      ->setDisplayOptions('view', array(
        'type' => 'hidden',
      ))
      ->setDisplayOptions('form', array(
        'type' => 'language_select',
        'weight' => 2,
      ));

    $fields['strapline'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Strapline'))
      ->setRequired(TRUE)
      ->setTranslatable(TRUE)
      ->setDisplayConfigurable('form', TRUE);

    $fields['body'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Details'))
      ->setRequired(FALSE)
      ->setTranslatable(TRUE)
      ->setSettings(['default_value' => ''])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Owner'))
      ->setDescription(t('The username of the small ad owner.'))
      ->setRevisionable(FALSE)
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback('Drupal\node\Entity\Node::getCurrentUserId')
      ->setTranslatable(TRUE)
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setDisplayOptions('view',[
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => array(
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'placeholder' => '',
        ),
      ]);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Last changed'))
      ->setDescription(t('When the ad was last saved.'))
      ->setRevisionable(FALSE)
      ->setTranslatable(FALSE)
      ->setDisplayOptions(
        'view',
        ['label' => 'hidden', 'type' => 'timestamp', 'weight' => 10]
      )
      ->setDisplayOptions(
        'form',
        ['type' => 'datetime_timestamp', 'weight' => 10]
      )
      ->setDisplayConfigurable('view', TRUE);

    $fields['scope'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Scope'))
      ->setDescription(t('How far afield this ad can be seen'))
      ->setSetting('min', SMALLAD_SCOPE_PRIVATE)
      ->setSetting('max', SMALLAD_SCOPE_PUBLIC)
      ->setDisplayConfigurable('form', TRUE)
      ->setDefaultValue(SMALLAD_SCOPE_SITE)
      ->setRequired(TRUE);

    //I tried to use the datetime fieldType but ...
    $fields['expires'] = BaseFieldDefinition::create('datetime')
      ->setLabel(t('Expires on'))
      ->setDescription(t('When the ad scope will revert to private'))
      ->setRevisionable(FALSE)
      ->setDisplayConfigurable('form', TRUE)
      ->setSetting('datetime_type', 'date')
      ->setDefaultValueCallback('Drupal\smallads\Entity\SmallAd::expiresDefault')
      ->setTranslatable(FALSE)
      ->setRequired(TRUE);

    $fields['attached'] = BaseFieldDefinition::create('file')
      ->setLabel(t('Image / Attachment'))
      ->setDescription(t('Any attached images or files'))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRevisionable(FALSE)
      ->setTranslatable(FALSE);

    $fields['uri'] = BaseFieldDefinition::create('uri')
      ->setLabel(t('Further information'))
      ->setDescription(t('Url of more information'))
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE)
      ->setRevisionable(FALSE)
      ->setTranslatable(TRUE);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  public static function expiresDefault(){//mtrace();
    $interval = \Drupal::config('smallads.conf')->get('default_expiry');
    //$values['expires'] = ['year'=> 1980, 'month'=> 8, 'day' => 8];//this isEmpty, unfortunately
    //$values['expires'] = '1999-9-9';//still not working...
    //return strtotime(\Drupal::config('smallads.conf')->get('default_expiry'));
    return DrupalDateTime::createFromTimestamp(strtotime($interval));

  }

  function expire() {
    //there might be an option on notification somewhere?
    $this->scope->value = SMALLAD_SCOPE_PRIVATE;
    $this->save();
    $account = $this->getOwner();
    //@todo test the expiry mail
    drupal_mail(
      'smallads',
      'expired',
      $account->getMail(),
      user_preferred_language($account),
      ['ad' => $ad, 'owner' => $account]
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getChangedTime() {
    return $this->get('changed')->value;
  }

}
